/**
 * \file main.c
 *
 * \brief TSYS02D temperature monitoring application file
 *
 * Copyright (c) 2014 Measurement Specialties. All rights reserved.
 *
 * \asf_license_start
 *
 * \page License
 *
 *
 * \asf_license_stop
 *
 */

/**
 * \mainpage TSYS02D driver Application documentation
 *
 * \par Description
 *
 * This is a basic application for monitoring TSYS02D temperature
 *
 * \par Content
 *
 * -# Include the ASF header files (through asf.h)
 * -# Minimal main function that starts with a call to system_init() and tsys02d_init()
 * -# Check presence of device : tsys02d_is_connected()
 * -# Reset tsys02d : tsys02d_reset()
 * -# Set temperature resolution : tsys02d_set_resolution()
 * -# Read tsys02d serial number : tsys02d_read_serial_number()
 * -# While loop that monitors temperature every s : tsys02d_read_temperature()
 *      - On each loop change the mode of access by calling tsys02d_set_i2c_master_mode()
 *      - During the loop, enable the heater for 10s using tsys02d_enable_heater() and come back to heater disabled with tsys02d_disable_heater()
 * 		- Retrieve the heater status  : tsys02d_get_heater_status()
 *      - When temperature increases, the LED0 will be switched on
 *      - When temperature decreases, the LED0 will be switched off
 */

/*
 * Include header files for all drivers that have been imported from
 * Atmel Software Framework (ASF).
 */
#include <asf.h>

float temperature;
uint64_t serial;

int main (void)
{
	enum tsys02d_status status;
	float last_temperature = 0;
	float variation = 0;
	uint8_t n=0;
	enum tsys02d_heater_status heater;
	
	system_init();
	delay_init();

	// Configure device and enable
	tsys02d_init();
	
	if( !tsys02d_is_connected() )
		return -1;
	
	// Reset TSYS02D
	status = tsys02d_reset();
	// Current HW does not acknowledge the data on I2C reset transfer
	//if( status != tsys02d_status_ok)
		//return -1;
	
	// Read serial number
	status = tsys02d_read_serial_number(&serial);
	if( status != tsys02d_status_ok)
		return -1;

	// Configure resolution
	status = tsys02d_set_resolution(tsys02d_resolution_t_14b);
	if( status != tsys02d_status_ok)
		return -1;

	// Monitor temperature every 500ms
	while (1) {
		
		// Enable heater for 10s
		if( (n==10) ) {
			status = tsys02d_enable_heater();
			if( status != tsys02d_status_ok)
				return -1;
		}
		// Disable heater after 20s
		if( (n==20) ) {
			status = tsys02d_disable_heater();
			if( status != tsys02d_status_ok)
				return -1;
		}
		
		// Check heater status
		status = tsys02d_get_heater_status(&heater);
		if( status != tsys02d_status_ok)
			return -1;
		
		// Alternate between w and w/o hold temperature read
		if( n & 1)
			tsys02d_set_i2c_master_mode(tsys02d_i2c_hold);
		else
			tsys02d_set_i2c_master_mode(tsys02d_i2c_no_hold);
			 
		tsys02d_read_temperature( &temperature);
		if( status != tsys02d_status_ok)
			return -1;
		
		variation += temperature - last_temperature;

		// Look for significant temperature variation
		if (  variation >= 0.5 ) {
			// Yes, so turn LED on.
			port_pin_set_output_level(LED_0_PIN, LED_0_ACTIVE);
			variation = 0;
		} else if ( variation <= -0.5 ) {
			// No, so turn LED off.
			port_pin_set_output_level(LED_0_PIN, LED_0_INACTIVE);
			variation = 0;
		}
		
		delay_ms(1000);
		last_temperature = temperature;
		n++;
		if(n==50) n=0;
	}
	
	return 0;
}
